/*
 ThermostatQA - test framework for Thermostat Monitoring Tool

 Copyright 2015 Red Hat, Inc.

 This file is part of ThermostatQA

 ThermostatQA is distributed under the GNU General Public License,
 version 2 or any later version (with a special exception described
 below, commonly known as the "Classpath Exception").

 A copy of GNU General Public License (GPL) is included in this
 distribution, in the file COPYING.

 Linking ThermostatQA code with other modules is making a combined work
 based on ThermostatQA.  Thus, the terms and conditions of the GPL
 cover the whole combination.

 As a special exception, the copyright holders of ThermostatQA give you
 permission to link this code with independent modules to produce an
 executable, regardless of the license terms of these independent
 modules, and to copy and distribute the resulting executable under
 terms of your choice, provided that you also meet, for each linked
 independent module, the terms and conditions of the license of that
 module.  An independent module is a module which is not derived from
 or based on ThermostatQA code.  If you modify ThermostatQA, you may
 extend this exception to your version of the software, but you are
 not obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version.
 */
package org.thermostat.qa2.framework.streamprocessors;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import org.thermostat.qa2.framework.SimpleLogger;

/**
 *
 * @author Zdeněk Žamberský
 */
public class BufferingStreamProcessor extends LoggingStreamProcessor {

    List<String> lineList = new ArrayList();
    boolean log;
    boolean closed = false;

    public BufferingStreamProcessor(InputStream is, SimpleLogger l, boolean log) {
        super(is, l);
        this.log = log;
    }

    @Override
    public void run() {
        InputStreamReader isr = new InputStreamReader(is);
        BufferedReader reader = new BufferedReader(isr);
        String line;
        try {
            while ((line = reader.readLine()) != null) {
                if (log) {
                    logger.log(line);
                }
                bufferLine(line);
            }
        } catch (IOException ex) {
        } finally {
            try {
                reader.close();
            } catch (IOException ex) {
            }
            synchronized(this){
                closed = true;
            }
        }
    }

    private synchronized void bufferLine(String line) {
        lineList.add(line);
    }

    public synchronized List<String> getLineList() throws IOException {
        List<String> list = lineList;
        if(!closed){
            lineList = new ArrayList();
        }
        return list;
    }

}
